@@ -20,6 +20,7 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
+	"io/ioutil"
 	"log"
 	"reflect"
 	"sort"
@@ -206,6 +207,80 @@ func testPromResourceUpdate(t *testing.T) {
 		t.Fatal(err)
 	}
 }
+func testPromStorageLabelsAnnotations(t *testing.T) {
+	t.Parallel()
+
+	ctx := framework.NewTestCtx(t)
+	defer ctx.Cleanup(t)
+	ns := ctx.CreateNamespace(t, framework.KubeClient)
+	ctx.SetupPrometheusRBAC(t, ns, framework.KubeClient)
+
+	name := "test"
+
+	p := framework.MakeBasicPrometheus(ns, name, name, 1)
+
+	p.Spec.Storage = &monitoringv1.StorageSpec{
+		VolumeClaimTemplate: monitoringv1.EmbeddedPersistentVolumeClaim{
+			EmbeddedObjectMetadata: monitoringv1.EmbeddedObjectMetadata{
+				Labels: map[string]string{
+					"test-label": "foo",
+				},
+				Annotations: map[string]string{
+					"test-annotation": "bar",
+				},
+			},
+			Spec: v1.PersistentVolumeClaimSpec{
+				AccessModes: []v1.PersistentVolumeAccessMode{v1.ReadWriteOnce},
+				Resources: v1.ResourceRequirements{
+					Requests: v1.ResourceList{
+						v1.ResourceStorage: resource.MustParse("200Mi"),
+					},
+				},
+			},
+		},
+	}
+
+	p, err := framework.CreatePrometheusAndWaitUntilReady(ns, p)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if val := p.Spec.Storage.VolumeClaimTemplate.Labels["test-label"]; val != "foo" {
+		t.Errorf("incorrect volume claim label, want: %v, got: %v", "foo", val)
+	}
+	if val := p.Spec.Storage.VolumeClaimTemplate.Annotations["test-annotation"]; val != "bar" {
+		t.Errorf("incorrect volume claim annoation, want: %v, got: %v", "bar", val)
+	}
+
+	err = wait.Poll(5*time.Second, 2*time.Minute, func() (bool, error) {
+		sts, err := framework.KubeClient.AppsV1().StatefulSets(ns).List(metav1.ListOptions{})
+		if err != nil {
+			return false, err
+		}
+
+		if len(sts.Items) < 1 {
+			return false, nil
+		}
+
+		for _, vct := range sts.Items[0].Spec.VolumeClaimTemplates {
+			if vct.Name == "prometheus-"+name+"-db" {
+				if val := vct.Labels["test-label"]; val != "foo" {
+					return false, errors.Errorf("incorrect volume claim label on sts, want: %v, got: %v", "foo", val)
+				}
+				if val := vct.Annotations["test-annotation"]; val != "bar" {
+					return false, errors.Errorf("incorrect volume claim annotation on sts, want: %v, got: %v", "bar", val)
+				}
+				return true, nil
+			}
+		}
+
+		return false, nil
+	})
+
+	if err != nil {
+		t.Fatal(err)
+	}
+}
 
 func testPromStorageUpdate(t *testing.T) {
 	t.Parallel()
@@ -225,10 +300,7 @@ func testPromStorageUpdate(t *testing.T) {
 	}
 
 	p.Spec.Storage = &monitoringv1.StorageSpec{
-		VolumeClaimTemplate: v1.PersistentVolumeClaim{
-			ObjectMeta: metav1.ObjectMeta{
-				CreationTimestamp: metav1.Now(),
-			},
+		VolumeClaimTemplate: monitoringv1.EmbeddedPersistentVolumeClaim{
 			Spec: v1.PersistentVolumeClaimSpec{
 				AccessModes: []v1.PersistentVolumeAccessMode{v1.ReadWriteOnce},
 				Resources: v1.ResourceRequirements{
@@ -477,7 +549,7 @@ func testPromAdditionalAlertManagerConfig(t *testing.T) {
 	}
 
 	err = wait.Poll(time.Second, 5*time.Minute, func() (done bool, err error) {
-		response, err := framework.QueryPrometheusSVC(ns, svc.Name, "/api/v1/alertmanagers", map[string]string{})
+		response, err := framework.PrometheusSVCGetRequest(ns, svc.Name, "/api/v1/alertmanagers", map[string]string{})
 		if err != nil {
 			return true, err
 		}
@@ -866,6 +938,17 @@ func testPromOnlyUpdatedOnRelevantChanges(t *testing.T) {
 			},
 			MaxExpectedChanges: 2,
 		},
+		{
+			Name: "tlsAssetSecret",
+			Getter: func(prometheusName string) (versionedResource, error) {
+				return framework.
+					KubeClient.
+					CoreV1().
+					Secrets(ns).
+					Get("prometheus-"+prometheusName+"-tls-assets", metav1.GetOptions{})
+			},
+			MaxExpectedChanges: 2,
+		},
 		{
 			Name: "statefulset",
 			Getter: func(prometheusName string) (versionedResource, error) {
@@ -958,7 +1041,7 @@ func testPromOnlyUpdatedOnRelevantChanges(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	err = isDiscoveryWorking(ns, pSVC.Name, prometheus.Name)
+	err = framework.WaitForDiscoveryWorking(ns, pSVC.Name, prometheus.Name)
 	if err != nil {
 		t.Fatal(errors.Wrap(err, "validating Prometheus target discovery failed"))
 	}
@@ -1060,7 +1143,7 @@ func testPromDiscovery(t *testing.T) {
 		t.Fatal("Generated Secret could not be retrieved: ", err)
 	}
 
-	err = isDiscoveryWorking(ns, svc.Name, prometheusName)
+	err = framework.WaitForDiscoveryWorking(ns, svc.Name, prometheusName)
 	if err != nil {
 		t.Fatal(errors.Wrap(err, "validating Prometheus target discovery failed"))
 	}
@@ -1197,7 +1280,7 @@ func testPromDiscoverTargetPort(t *testing.T) {
 		t.Fatal("Generated Secret could not be retrieved: ", err)
 	}
 
-	err = isDiscoveryWorking(ns, svc.Name, prometheusName)
+	err = framework.WaitForDiscoveryWorking(ns, svc.Name, prometheusName)
 	if err != nil {
 		t.Fatal(errors.Wrap(err, "validating Prometheus target discovery failed"))
 	}
@@ -1249,7 +1332,7 @@ func testPromOpMatchPromAndServMonInDiffNSs(t *testing.T) {
 		ctx.AddFinalizerFn(finalizerFn)
 	}
 
-	resp, err := framework.QueryPrometheusSVC(prometheusNSName, svc.Name, "/api/v1/status/config", map[string]string{})
+	resp, err := framework.PrometheusSVCGetRequest(prometheusNSName, svc.Name, "/api/v1/status/config", map[string]string{})
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -1353,105 +1436,146 @@ func testThanos(t *testing.T) {
 	}
 }
 
-func testPromGetBasicAuthSecret(t *testing.T) {
+func testPromGetAuthSecret(t *testing.T) {
 	t.Parallel()
-
-	ctx := framework.NewTestCtx(t)
-	defer ctx.Cleanup(t)
-	ns := ctx.CreateNamespace(t, framework.KubeClient)
-	ctx.SetupPrometheusRBACGlobal(t, ns, framework.KubeClient)
-
 	name := "test"
 
-	maptest := make(map[string]string)
-	maptest["tc"] = ns
-	prometheusCRD := framework.MakeBasicPrometheus(ns, name, name, 1)
-	prometheusCRD.Spec.ServiceMonitorNamespaceSelector = &metav1.LabelSelector{
-		MatchLabels: maptest,
-	}
-
-	if _, err := framework.CreatePrometheusAndWaitUntilReady(ns, prometheusCRD); err != nil {
-		t.Fatal(err)
-	}
-	testNamespace := ctx.CreateNamespace(t, framework.KubeClient)
-
-	err := testFramework.AddLabelsToNamespace(framework.KubeClient, testNamespace, maptest)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	simple, err := testFramework.MakeDeployment("../../test/framework/ressources/basic-auth-app-deployment.yaml")
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	if err := testFramework.CreateDeployment(framework.KubeClient, testNamespace, simple); err != nil {
-		t.Fatal("Creating simple basic auth app failed: ", err)
-	}
-
-	authSecret := &v1.Secret{
-		ObjectMeta: metav1.ObjectMeta{
-			Name: name,
-		},
-		Data: map[string][]byte{
-			"user":     []byte("user"),
-			"password": []byte("pass"),
-		},
-	}
-
-	if _, err := framework.KubeClient.CoreV1().Secrets(testNamespace).Create(authSecret); err != nil {
-		t.Fatal(err)
-	}
-
-	svc := &v1.Service{
-		ObjectMeta: metav1.ObjectMeta{
-			Name: name,
-			Labels: map[string]string{
-				"group": name,
-			},
-		},
-		Spec: v1.ServiceSpec{
-			Type: v1.ServiceTypeLoadBalancer,
-			Ports: []v1.ServicePort{
-				v1.ServicePort{
-					Name: "web",
-					Port: 8080,
+	tests := []struct {
+		name           string
+		secret         *v1.Secret
+		serviceMonitor func() *monitoringv1.ServiceMonitor
+	}{
+		{
+			name: "basic-auth",
+			secret: &v1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: name,
+				},
+				Data: map[string][]byte{
+					"user":     []byte("user"),
+					"password": []byte("pass"),
 				},
 			},
-			Selector: map[string]string{
-				"group": name,
+			serviceMonitor: func() *monitoringv1.ServiceMonitor {
+				sm := framework.MakeBasicServiceMonitor(name)
+				sm.Spec.Endpoints[0].BasicAuth = &monitoringv1.BasicAuth{
+					Username: v1.SecretKeySelector{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: name,
+						},
+						Key: "user",
+					},
+					Password: v1.SecretKeySelector{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: name,
+						},
+						Key: "password",
+					},
+				}
+
+				return sm
+			},
+		},
+		{
+			name: "bearer-token",
+			secret: &v1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: name,
+				},
+				Data: map[string][]byte{
+					"bearertoken": []byte("abc"),
+				},
+			},
+			serviceMonitor: func() *monitoringv1.ServiceMonitor {
+				sm := framework.MakeBasicServiceMonitor(name)
+				sm.Spec.Endpoints[0].BearerTokenSecret = v1.SecretKeySelector{
+					LocalObjectReference: v1.LocalObjectReference{
+						Name: name,
+					},
+					Key: "bearertoken",
+				}
+				sm.Spec.Endpoints[0].Path = "/bearer-token"
+				return sm
 			},
 		},
 	}
 
-	sm := framework.MakeBasicServiceMonitor(name)
-	sm.Spec.Endpoints[0].BasicAuth = &monitoringv1.BasicAuth{
-		Username: v1.SecretKeySelector{
-			LocalObjectReference: v1.LocalObjectReference{
-				Name: name,
-			},
-			Key: "user",
-		},
-		Password: v1.SecretKeySelector{
-			LocalObjectReference: v1.LocalObjectReference{
-				Name: name,
-			},
-			Key: "password",
-		},
-	}
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			t.Parallel()
+			ctx := framework.NewTestCtx(t)
+			defer ctx.Cleanup(t)
+			ns := ctx.CreateNamespace(t, framework.KubeClient)
+			ctx.SetupPrometheusRBACGlobal(t, ns, framework.KubeClient)
 
-	if finalizerFn, err := testFramework.CreateServiceAndWaitUntilReady(framework.KubeClient, testNamespace, svc); err != nil {
-		t.Fatal(err)
-	} else {
-		ctx.AddFinalizerFn(finalizerFn)
-	}
+			maptest := make(map[string]string)
+			maptest["tc"] = ns
+			prometheusCRD := framework.MakeBasicPrometheus(ns, name, name, 1)
+			prometheusCRD.Spec.ServiceMonitorNamespaceSelector = &metav1.LabelSelector{
+				MatchLabels: maptest,
+			}
+			prometheusCRD.Spec.ScrapeInterval = "1s"
 
-	if _, err := framework.MonClientV1.ServiceMonitors(testNamespace).Create(sm); err != nil {
-		t.Fatal("Creating ServiceMonitor failed: ", err)
-	}
+			if _, err := framework.CreatePrometheusAndWaitUntilReady(ns, prometheusCRD); err != nil {
+				t.Fatal(err)
+			}
+			testNamespace := ctx.CreateNamespace(t, framework.KubeClient)
 
-	if err := framework.WaitForTargets(ns, "prometheus-operated", 1); err != nil {
-		t.Fatal(err)
+			err := testFramework.AddLabelsToNamespace(framework.KubeClient, testNamespace, maptest)
+			if err != nil {
+				t.Fatal(err)
+			}
+
+			simple, err := testFramework.MakeDeployment("../../test/framework/ressources/basic-auth-app-deployment.yaml")
+			if err != nil {
+				t.Fatal(err)
+			}
+
+			if err := testFramework.CreateDeployment(framework.KubeClient, testNamespace, simple); err != nil {
+				t.Fatal("Creating simple basic auth app failed: ", err)
+			}
+
+			authSecret := test.secret
+			if _, err := framework.KubeClient.CoreV1().Secrets(testNamespace).Create(authSecret); err != nil {
+				t.Fatal(err)
+			}
+
+			svc := &v1.Service{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: name,
+					Labels: map[string]string{
+						"group": name,
+					},
+				},
+				Spec: v1.ServiceSpec{
+					Type: v1.ServiceTypeLoadBalancer,
+					Ports: []v1.ServicePort{
+						v1.ServicePort{
+							Name: "web",
+							Port: 8080,
+						},
+					},
+					Selector: map[string]string{
+						"group": name,
+					},
+				},
+			}
+
+			sm := test.serviceMonitor()
+			if finalizerFn, err := testFramework.CreateServiceAndWaitUntilReady(framework.KubeClient, testNamespace, svc); err != nil {
+				t.Fatal(err)
+			} else {
+				ctx.AddFinalizerFn(finalizerFn)
+			}
+
+			if _, err := framework.MonClientV1.ServiceMonitors(testNamespace).Create(sm); err != nil {
+				t.Fatal("Creating ServiceMonitor failed: ", err)
+			}
+
+			if err := framework.WaitForTargets(ns, "prometheus-operated", 1); err != nil {
+				t.Fatal(err)
+			}
+		})
 	}
 
 }
@@ -1487,7 +1611,7 @@ func testOperatorNSScope(t *testing.T) {
 		}
 
 		// Prometheus Operator only watches single namespace mainNS, not arbitraryNS.
-		_, err := framework.CreatePrometheusOperator(operatorNS, *opImage, []string{mainNS}, nil, false)
+		_, err := framework.CreatePrometheusOperator(operatorNS, *opImage, []string{mainNS}, nil, nil, nil, false)
 		if err != nil {
 			t.Fatal(err)
 		}
@@ -1557,7 +1681,7 @@ func testOperatorNSScope(t *testing.T) {
 		}
 
 		// Prometheus Operator only watches prometheusNS and ruleNS, not arbitraryNS.
-		_, err := framework.CreatePrometheusOperator(operatorNS, *opImage, []string{prometheusNS, ruleNS}, nil, false)
+		_, err := framework.CreatePrometheusOperator(operatorNS, *opImage, []string{prometheusNS, ruleNS}, nil, nil, nil, false)
 		if err != nil {
 			t.Fatal(err)
 		}
@@ -1607,79 +1731,525 @@ func testOperatorNSScope(t *testing.T) {
 	})
 }
 
-func isDiscoveryWorking(ns, svcName, prometheusName string) error {
-	var loopErr error
+// testPromArbitraryFSAcc tests the
+// github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1.PrometheusSpec.ArbitraryFSAccessThroughSMs
+// configuration with the service monitor bearer token and tls assets option.
+func testPromArbitraryFSAcc(t *testing.T) {
+	t.Parallel()
 
-	err := wait.Poll(time.Second, 5*framework.DefaultTimeout, func() (bool, error) {
-		pods, loopErr := framework.KubeClient.CoreV1().Pods(ns).List(prometheus.ListOptions(prometheusName))
-		if loopErr != nil {
-			return false, loopErr
-		}
-		if 1 != len(pods.Items) {
-			return false, nil
-		}
-		podIP := pods.Items[0].Status.PodIP
-		expectedTargets := []string{fmt.Sprintf("http://%s:9090/metrics", podIP)}
+	name := "test"
 
-		activeTargets, loopErr := framework.GetActiveTargets(ns, svcName)
-		if loopErr != nil {
-			return false, loopErr
-		}
+	tests := []struct {
+		name                              string
+		arbitraryFSAccessThroughSMsConfig monitoringv1.ArbitraryFSAccessThroughSMsConfig
+		endpoint                          monitoringv1.Endpoint
+		expectTargets                     bool
+	}{
+		//
+		// Bearer tokens:
+		//
+		{
+			name: "allowed-bearer-file",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: false,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port:            "web",
+				BearerTokenFile: "abc",
+			},
+			expectTargets: true,
+		},
+		{
+			name: "denied-bearer-file",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: true,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port:            "web",
+				BearerTokenFile: "abc",
+			},
+			expectTargets: false,
+		},
+		{
+			name: "allowed-bearer-secret",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: false,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port: "web",
+				BearerTokenSecret: v1.SecretKeySelector{
+					LocalObjectReference: v1.LocalObjectReference{
+						Name: name,
+					},
+					Key: "bearer-token",
+				},
+			},
+			expectTargets: true,
+		},
+		{
+			name: "denied-bearer-secret",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: true,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port: "web",
+				BearerTokenSecret: v1.SecretKeySelector{
+					LocalObjectReference: v1.LocalObjectReference{
+						Name: name,
+					},
+					Key: "bearer-token",
+				},
+			},
+			expectTargets: true,
+		},
+		//
+		// TLS assets:
+		//
+		{
+			name: "allowed-tls-file",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: false,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port: "web",
+				TLSConfig: &monitoringv1.TLSConfig{
+					CAFile:   "/etc/ca-certificates/example-ca.pem",
+					CertFile: "/etc/ca-certificates/example-cert.pem",
+					KeyFile:  "/etc/ca-certificates/example-key.pem",
+				},
+			},
+			expectTargets: true,
+		},
+		{
+			name: "denied-tls-file",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: true,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port: "web",
+				TLSConfig: &monitoringv1.TLSConfig{
+					CAFile:   "/etc/ca-certificates/example-ca.pem",
+					CertFile: "/etc/ca-certificates/example-cert.pem",
+					KeyFile:  "/etc/ca-certificates/example-key.pem",
+				},
+			},
+			expectTargets: false,
+		},
+		{
+			name: "allowed-tls-secret",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: false,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port: "web",
+				TLSConfig: &monitoringv1.TLSConfig{
+					InsecureSkipVerify: true,
+					CA: monitoringv1.SecretOrConfigMap{
+						Secret: &v1.SecretKeySelector{
+							LocalObjectReference: v1.LocalObjectReference{
+								Name: name,
+							},
+							Key: "cert.pem",
+						},
+					},
+					Cert: monitoringv1.SecretOrConfigMap{
+						Secret: &v1.SecretKeySelector{
+							LocalObjectReference: v1.LocalObjectReference{
+								Name: name,
+							},
+							Key: "cert.pem",
+						},
+					},
+					KeySecret: &v1.SecretKeySelector{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: name,
+						},
+						Key: "key.pem",
+					},
+				},
+			},
+			expectTargets: true,
+		},
+		{
+			name: "denied-tls-secret",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: true,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port: "web",
+				TLSConfig: &monitoringv1.TLSConfig{
+					InsecureSkipVerify: true,
+					CA: monitoringv1.SecretOrConfigMap{
+						Secret: &v1.SecretKeySelector{
+							LocalObjectReference: v1.LocalObjectReference{
+								Name: name,
+							},
+							Key: "cert.pem",
+						},
+					},
+					Cert: monitoringv1.SecretOrConfigMap{
+						Secret: &v1.SecretKeySelector{
+							LocalObjectReference: v1.LocalObjectReference{
+								Name: name,
+							},
+							Key: "cert.pem",
+						},
+					},
+					KeySecret: &v1.SecretKeySelector{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: name,
+						},
+						Key: "key.pem",
+					},
+				},
+			},
+			expectTargets: false,
+		},
+		{
+			name: "allowed-tls-configmap",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: false,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port: "web",
+				TLSConfig: &monitoringv1.TLSConfig{
+					InsecureSkipVerify: true,
+					CA: monitoringv1.SecretOrConfigMap{
+						ConfigMap: &v1.ConfigMapKeySelector{
+							LocalObjectReference: v1.LocalObjectReference{
+								Name: name,
+							},
+							Key: "cert.pem",
+						},
+					},
+					Cert: monitoringv1.SecretOrConfigMap{
+						ConfigMap: &v1.ConfigMapKeySelector{
+							LocalObjectReference: v1.LocalObjectReference{
+								Name: name,
+							},
+							Key: "cert.pem",
+						},
+					},
+					KeySecret: &v1.SecretKeySelector{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: name,
+						},
+						Key: "key.pem",
+					},
+				},
+			},
+			expectTargets: true,
+		},
+		{
+			name: "denied-tls-configmap",
+			arbitraryFSAccessThroughSMsConfig: monitoringv1.ArbitraryFSAccessThroughSMsConfig{
+				Deny: true,
+			},
+			endpoint: monitoringv1.Endpoint{
+				Port: "web",
+				TLSConfig: &monitoringv1.TLSConfig{
+					InsecureSkipVerify: true,
+					CA: monitoringv1.SecretOrConfigMap{
+						ConfigMap: &v1.ConfigMapKeySelector{
+							LocalObjectReference: v1.LocalObjectReference{
+								Name: name,
+							},
+							Key: "cert.pem",
+						},
+					},
+					Cert: monitoringv1.SecretOrConfigMap{
+						ConfigMap: &v1.ConfigMapKeySelector{
+							LocalObjectReference: v1.LocalObjectReference{
+								Name: name,
+							},
+							Key: "cert.pem",
+						},
+					},
+					KeySecret: &v1.SecretKeySelector{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: name,
+						},
+						Key: "key.pem",
+					},
+				},
+			},
+			expectTargets: false,
+		},
+	}
 
-		if loopErr = assertExpectedTargets(activeTargets, expectedTargets); loopErr != nil {
-			return false, nil
-		}
+	for _, test := range tests {
+		test := test
 
-		working, loopErr := basicQueryWorking(ns, svcName)
-		if loopErr != nil {
-			return false, loopErr
-		}
-		if !working {
-			return false, nil
-		}
+		t.Run(test.name, func(t *testing.T) {
+			t.Parallel()
 
-		return true, nil
-	})
+			ctx := framework.NewTestCtx(t)
+			defer ctx.Cleanup(t)
+			ns := ctx.CreateNamespace(t, framework.KubeClient)
+			ctx.SetupPrometheusRBAC(t, ns, framework.KubeClient)
 
+			// Create secret either used by bearer token secret key ref, tls
+			// asset key ref or tls configmap key ref.
+			cert, err := ioutil.ReadFile("../../test/instrumented-sample-app/certs/cert.pem")
+			if err != nil {
+				t.Fatalf("failed to load cert.pem: %v", err)
+			}
+
+			key, err := ioutil.ReadFile("../../test/instrumented-sample-app/certs/key.pem")
+			if err != nil {
+				t.Fatalf("failed to load key.pem: %v", err)
+			}
+
+			tlsCertsSecret := &v1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: name,
+				},
+				Data: map[string][]byte{
+					"cert.pem":     cert,
+					"key.pem":      key,
+					"bearer-token": []byte("abc"),
+				},
+			}
+
+			if _, err := framework.KubeClient.CoreV1().Secrets(ns).Create(tlsCertsSecret); err != nil {
+				t.Fatal(err)
+			}
+
+			tlsCertsConfigMap := &v1.ConfigMap{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: name,
+				},
+				Data: map[string]string{
+					"cert.pem": string(cert),
+				},
+			}
+
+			if _, err := framework.KubeClient.CoreV1().ConfigMaps(ns).Create(tlsCertsConfigMap); err != nil {
+				t.Fatal(err)
+			}
+
+			prometheusCRD := framework.MakeBasicPrometheus(ns, name, name, 1)
+			prometheusCRD.Namespace = ns
+			prometheusCRD.Spec.ArbitraryFSAccessThroughSMs = test.arbitraryFSAccessThroughSMsConfig
+
+			if _, err := framework.CreatePrometheusAndWaitUntilReady(ns, prometheusCRD); err != nil {
+				t.Fatal(err)
+			}
+
+			svc := framework.MakePrometheusService(prometheusCRD.Name, name, v1.ServiceTypeClusterIP)
+			if _, err := testFramework.CreateServiceAndWaitUntilReady(framework.KubeClient, ns, svc); err != nil {
+				t.Fatal(err)
+			}
+
+			s := framework.MakeBasicServiceMonitor(name)
+			s.Spec.Endpoints[0] = test.endpoint
+			if _, err := framework.MonClientV1.ServiceMonitors(ns).Create(s); err != nil {
+				t.Fatal("creating ServiceMonitor failed: ", err)
+			}
+
+			if test.expectTargets {
+				if err := framework.WaitForTargets(ns, svc.Name, 1); err != nil {
+					t.Fatal(err)
+				}
+
+				return
+			}
+
+			// Make sure Prometheus has enough time to reload.
+			time.Sleep(2 * time.Minute)
+			if err := framework.WaitForTargets(ns, svc.Name, 0); err != nil {
+				t.Fatal(err)
+			}
+		})
+	}
+
+}
+
+// testPromTLSConfigViaSecret tests the service monitor endpoint option to load
+// certificate assets via Kubernetes secrets into the Prometheus container.
+func testPromTLSConfigViaSecret(t *testing.T) {
+	t.Parallel()
+
+	ctx := framework.NewTestCtx(t)
+	defer ctx.Cleanup(t)
+
+	ns := ctx.CreateNamespace(t, framework.KubeClient)
+	ctx.SetupPrometheusRBAC(t, ns, framework.KubeClient)
+	name := "test"
+
+	//
+	// Setup sample app.
+	//
+
+	cert, err := ioutil.ReadFile("../../test/instrumented-sample-app/certs/cert.pem")
 	if err != nil {
-		return fmt.Errorf("waiting for Prometheus to discover targets failed: %v: %v", err, loopErr)
+		t.Fatalf("failed to load cert.pem: %v", err)
 	}
 
-	return nil
-}
-
-type resultVector struct {
-	Metric map[string]string `json:"metric"`
-	Value  []interface{}     `json:"value"`
-}
-
-type queryResult struct {
-	ResultType string          `json:"resultType"`
-	Result     []*resultVector `json:"result"`
-}
-
-type prometheusQueryAPIResponse struct {
-	Status string       `json:"status"`
-	Data   *queryResult `json:"data"`
-}
-
-func basicQueryWorking(ns, svcName string) (bool, error) {
-	response, err := framework.QueryPrometheusSVC(ns, svcName, "/api/v1/query", map[string]string{"query": "up"})
+	key, err := ioutil.ReadFile("../../test/instrumented-sample-app/certs/key.pem")
 	if err != nil {
-		return false, err
+		t.Fatalf("failed to load key.pem: %v", err)
 	}
 
-	rq := prometheusQueryAPIResponse{}
-	if err := json.NewDecoder(bytes.NewBuffer(response)).Decode(&rq); err != nil {
-		return false, err
+	tlsCertsSecret := &v1.Secret{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: name,
+		},
+		Data: map[string][]byte{
+			"cert.pem": cert,
+			"key.pem":  key,
+		},
 	}
 
-	if rq.Status != "success" && rq.Data.Result[0].Value[1] == "1" {
-		log.Printf("Query Response not successful.")
-		return false, nil
+	if _, err := framework.KubeClient.CoreV1().Secrets(ns).Create(tlsCertsSecret); err != nil {
+		t.Fatal(err)
 	}
 
-	return true, nil
+	simple, err := testFramework.MakeDeployment("../../test/framework/ressources/basic-auth-app-deployment.yaml")
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	simple.Spec.Template.Spec.Containers[0].Args = []string{"--cert-path=/etc/certs"}
+
+	simple.Spec.Template.Spec.Volumes = []v1.Volume{
+		v1.Volume{
+			Name: "tls-certs",
+			VolumeSource: v1.VolumeSource{
+				Secret: &v1.SecretVolumeSource{
+					SecretName: tlsCertsSecret.Name,
+				},
+			},
+		},
+	}
+
+	simple.Spec.Template.Spec.Containers[0].VolumeMounts = []v1.VolumeMount{
+		{
+			Name:      simple.Spec.Template.Spec.Volumes[0].Name,
+			MountPath: "/etc/certs",
+		},
+	}
+
+	if err := testFramework.CreateDeployment(framework.KubeClient, ns, simple); err != nil {
+		t.Fatal("Creating simple basic auth app failed: ", err)
+	}
+
+	svc := &v1.Service{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: name,
+			Labels: map[string]string{
+				"group": name,
+			},
+		},
+		Spec: v1.ServiceSpec{
+			Type: v1.ServiceTypeLoadBalancer,
+			Ports: []v1.ServicePort{
+				v1.ServicePort{
+					Name: "web",
+					Port: 8080,
+				},
+				v1.ServicePort{
+					Name: "mtls",
+					Port: 8081,
+				},
+			},
+			Selector: map[string]string{
+				"group": name,
+			},
+		},
+	}
+
+	if _, err := testFramework.CreateServiceAndWaitUntilReady(framework.KubeClient, ns, svc); err != nil {
+		t.Fatal(err)
+	}
+
+	//
+	// Setup monitoring.
+	//
+
+	sm := framework.MakeBasicServiceMonitor(name)
+	sm.Spec.Endpoints = []monitoringv1.Endpoint{
+		{
+			Port:     "mtls",
+			Interval: "30s",
+			Scheme:   "https",
+			TLSConfig: &monitoringv1.TLSConfig{
+				InsecureSkipVerify: true,
+				CA: monitoringv1.SecretOrConfigMap{
+					Secret: &v1.SecretKeySelector{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: tlsCertsSecret.Name,
+						},
+						Key: "cert.pem",
+					},
+				},
+				Cert: monitoringv1.SecretOrConfigMap{
+					Secret: &v1.SecretKeySelector{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: tlsCertsSecret.Name,
+						},
+						Key: "cert.pem",
+					},
+				},
+				KeySecret: &v1.SecretKeySelector{
+					LocalObjectReference: v1.LocalObjectReference{
+						Name: tlsCertsSecret.Name,
+					},
+					Key: "key.pem",
+				},
+			},
+		},
+	}
+
+	if _, err := framework.MonClientV1.ServiceMonitors(ns).Create(sm); err != nil {
+		t.Fatal("creating ServiceMonitor failed: ", err)
+	}
+
+	prometheusCRD := framework.MakeBasicPrometheus(ns, name, name, 1)
+
+	if _, err := framework.CreatePrometheusAndWaitUntilReady(ns, prometheusCRD); err != nil {
+		t.Fatal(err)
+	}
+
+	promSVC := framework.MakePrometheusService(prometheusCRD.Name, name, v1.ServiceTypeClusterIP)
+
+	if _, err := testFramework.CreateServiceAndWaitUntilReady(framework.KubeClient, ns, promSVC); err != nil {
+		t.Fatal(err)
+	}
+
+	//
+	// Check for proper scraping.
+	//
+
+	if err := framework.WaitForTargets(ns, promSVC.Name, 1); err != nil {
+		t.Fatal(err)
+	}
+
+	// TODO: Do a poll instead, should speed up things.
+	time.Sleep(30 * time.Second)
+
+	response, err := framework.PrometheusSVCGetRequest(
+		ns,
+		promSVC.Name,
+		"/api/v1/query",
+		map[string]string{"query": fmt.Sprintf(`up{job="%v",endpoint="%v"}`, name, sm.Spec.Endpoints[0].Port)},
+	)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	q := testFramework.PrometheusQueryAPIResponse{}
+	if err := json.NewDecoder(bytes.NewBuffer(response)).Decode(&q); err != nil {
+		t.Fatal(err)
+	}
+
+	if q.Status != "success" {
+		t.Fatalf("expected query status to be 'success' but got %v", q.Status)
+	}
+
+	if q.Data.Result[0].Value[1] != "1" {
+		t.Fatalf("expected query result to be '1' but got %v", q.Data.Result[0].Value[1])
+	}
 }
 
 func isAlertmanagerDiscoveryWorking(ns, promSVCName, alertmanagerName string) func() (bool, error) {
@@ -1696,7 +2266,7 @@ func isAlertmanagerDiscoveryWorking(ns, promSVCName, alertmanagerName string) fu
 			expectedAlertmanagerTargets = append(expectedAlertmanagerTargets, fmt.Sprintf("http://%s:9093/api/v1/alerts", p.Status.PodIP))
 		}
 
-		response, err := framework.QueryPrometheusSVC(ns, promSVCName, "/api/v1/alertmanagers", map[string]string{})
+		response, err := framework.PrometheusSVCGetRequest(ns, promSVCName, "/api/v1/alertmanagers", map[string]string{})
 		if err != nil {
 			return false, err
 		}
@@ -1714,26 +2284,6 @@ func isAlertmanagerDiscoveryWorking(ns, promSVCName, alertmanagerName string) fu
 	}
 }
 
-func assertExpectedTargets(targets []*testFramework.Target, expectedTargets []string) error {
-	existingTargets := []string{}
-
-	for _, t := range targets {
-		existingTargets = append(existingTargets, t.ScrapeURL)
-	}
-
-	sort.Strings(expectedTargets)
-	sort.Strings(existingTargets)
-
-	if !reflect.DeepEqual(expectedTargets, existingTargets) {
-		return fmt.Errorf(
-			"expected targets %q but got %q", strings.Join(expectedTargets, ","),
-			strings.Join(existingTargets, ","),
-		)
-	}
-
-	return nil
-}
-
 func assertExpectedAlertmanagerTargets(ams []*alertmanagerTarget, expectedTargets []string) bool {
 	log.Printf("Expected Alertmanager Targets: %#+v\n", expectedTargets)
 
